@@ -2,7 +2,7 @@ use warnings;
 use strict;
 
 package Mail::IMAPClient;
-our $VERSION = '3.03';
+our $VERSION = '3.05';
 
 use Mail::IMAPClient::MessageSet;
 
@@ -1566,11 +1566,12 @@ sub get_bodystructure
     }
     else
     {   $self->_debug("get_bodystructure: reassembling original response");
-        my $start = 0;
-        foreach my $o ($self->Results)
+        my $started = 0;
+        my $output  = '';
+        foreach my $o ($self->_transaction)
         {   next unless $self->_is_output_or_literal($o);
-            next unless $start or
-                $o->[DATA] =~ /BODYSTRUCTURE \(/i and ++$start; # Hi, vi! ;-)
+            $started++ if $o->[DATA] =~ /BODYSTRUCTURE \(/i; ; # Hi, vi! ;-)
+            $started or next;
 
             if(length $output && $self->_is_literal($o) )
             {   my $data = $o->[DATA];
@@ -1598,24 +1599,30 @@ sub get_envelope
         return undef;
     }
 
-    my @out = $self->fetch($msg,"ENVELOPE");
+    my @out = $self->fetch($msg, 'ENVELOPE');
     my $bs = "";
-    my $output = first { /ENVELOPE \(/i } @out;    # Wee! ;-)
+    my $output = first { /ENVELOPE \(/i } @out;    # vi ;-)
+
+    unless($output)
+    {   $self->LastError("Unable to use get_envelope: $@");
+        return undef;
+    }
+
     if($output =~ /\r\n$/ )
     {   eval { $bs = Mail::IMAPClient::BodyStructure::Envelope->new($output) };
     }
     else
     {   $self->_debug("get_envelope: reassembling original response");
-        my $start = 0;
-        foreach my $o ($self->Results)
+        my $started = 0;
+        $output = '';
+        foreach my $o ($self->_transaction)
         {   next unless $self->_is_output_or_literal($o);
             $self->_debug("o->[DATA] is $o->[DATA]");
 
-            next unless $start or
-                $o->[DATA] =~ /ENVELOPE \(/i and ++$start;
-                # Hi, vi! ;-)
+            $started++ if $o->[DATA] =~ /ENVELOPE \(/i; # Hi, vi! ;-)
+            $started or next;
 
-            if ( length($output) and $self->_is_literal($o) ) {
+            if(length($output) && $self->_is_literal($o) ) {
                 my $data = $o->[DATA];
                 $data =~ s/"/\\"/g;
                 $data =~ s/\(/\\\(/g;
@@ -1864,27 +1871,36 @@ sub parse_headers
     my @raw = $self->fetch($string)
         or return undef;
 
-    my %headers; # HASH from message ids to headers
-    my $h;       # HASH of fields for current msgid
-    my $field;   # previous field name
+    my %headers; # message ids to headers
+    my $h;       # fields for current msgid
+    my $field;   # previous field name, for unfolding
     my %fieldmap = map { ( lc($_) => $_ ) } @fields;
+    my $msgid;
 
     foreach my $header (map {split /\r?\n/} @raw)
-    {
+    {   # little problem: Windows2003 has UID as body, not in header
         if($header =~ s/^\* \s+ (\d+) \s+ FETCH \s+
                         \( (.*?) BODY\[HEADER (?:\.FIELDS)? .*? \]\s*//ix)
         {   # start new message header
-            my ($msgid, $msgattrs) = ($1, $2);
-            $msgid = $1 if $self->Uid && $msgattrs =~ m/\b UID \s+ (\d+)/x;
-            $h = $headers{$msgid} = {};
+            ($msgid, my $msgattrs) = ($1, $2);
+            $h = {};
+            if($self->Uid)  # undef when win2003
+            {   $msgid = $msgattrs =~ m/\b UID \s+ (\d+)/x ? $1 : undef }
+
+            $headers{$msgid} = $h if $msgid;
         }
-        $header =~ /\S/ or next;
+        $header =~ /\S/ or next; # skip empty lines.
 
         # ( for vi
         if($header =~ /^\)/)  # end of this message
         {   undef $h;  # inbetween headers
             next;
         }
+        elsif(!$msgid && $header =~ /^\s*UID\s+(\d+)\s*\)/)
+        {   $headers{$1} = $h;   # finally found msgid, win2003
+            undef $h;
+            next;
+        }
 
         unless(defined $h)
         {   last if $header =~ / OK /i;
@@ -2485,10 +2501,12 @@ sub authenticate
     until($code)
     {   my $output = $self->_read_line or return undef;
         foreach my $o (@$output)
-        {   $self->_record($count,$o);
-            $code = $o->[DATA] =~ /^\+\s+(.*)$/ ? $1 : undef;
+        {   $self->_record($count, $o);
+            $code = $o->[DATA] =~ /^\+\s+(\S+)\s*$/ ? $1
+                  : $o->[DATA] =~ /^\+\s*$/        ? 'OK'
+                  : undef;
 
-            if ($o->[DATA] =~ /^\*\s+BYE/)
+            if($o->[DATA] =~ /^\*\s+BYE/)
             {   $self->State(Unconnected);
                 return undef;
             }
@@ -2554,12 +2572,12 @@ sub authenticate
         return undef;
     }
 
-    undef $code = $scheme eq 'PLAIN' ? 'OK' : undef;
+    undef $code;
     until($code)
     {   my $output = $self->_read_line or return undef;
         foreach my $o (@$output)
         {   $self->_record($count, $o);
-            $code = $o->[DATA] =~ /^\+\s+(.*)$/ ? $1 : undef;
+            $code = $o->[DATA] =~ /^\+\s+(.*?)\s*$/ ? $1 : undef;
 
             if($code)
             {   unless($self->_send_line($response->($code, $self)))
